// Copyright (C) 2005-2007 CEREA
//     Author(s): Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea/) is a joint laboratory of ENPC
// (http://www.enpc.fr/) and EDF R&D (http://www.edf.fr/).
//
// This file is part of the air quality modeling system Polyphemus.
//
// Polyphemus is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// Polyphemus is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// For more information, please see the Polyphemus web site:
//     http://www.enpc.fr/cerea/polyphemus/


#ifndef POLYPHEMUS_FILE_DRIVER_STATIONARYDRIVER_CXX


#include "StationaryDriver.hxx"


namespace Polyphemus
{


  //! Main constructor.
  /*! Builds the driver and reads option keys in the configuration file.
    \param config_file configuration file.
   */
  template<class T, class ClassModel, class ClassOutputSaver>
  StationaryDriver<T, ClassModel, ClassOutputSaver>
  ::StationaryDriver(string config_file):
    BaseDriver<T, ClassModel, ClassOutputSaver>(config_file)
  {

    /*** Stationary conditions ***/

    this->config.SetSection("[stationary]");
    this->config.GetValue("Delta_t", Delta_t);
    this->config.GetValue("Nt", Nt);
  }

  
  //! Destructor.
  template<class T, class ClassModel, class ClassOutputSaver>
  StationaryDriver<T, ClassModel, ClassOutputSaver>::~StationaryDriver()
  {
  }


  //! Performs the simulation.
  /*! Initializes the model and the output saver, and then performs the time
    loop with calls to the model and to the output saver.
   */
  template<class T, class ClassModel, class ClassOutputSaver>
  void StationaryDriver<T, ClassModel, ClassOutputSaver>::Run()
  {

    /*** Initializations ***/

    this->Model.Init();
    this->OutputSaver.Init(this->Model);

    /*** Main time loop ***/

    Date date = this->Model.GetCurrentDate();
    for (int h = 0; h < Nt; h++)
      {
	
	if (this->option_display["show_iterations"])
	  cout << "Performing iteration #" << h << " at date "
	       << date.GetDate("%y-%m-%d %h:%i") << endl;

	this->Model.SetDate(date);
	this->Model.InitStep();

	this->OutputSaver.InitStep(this->Model);
	
	/*** Time inner-loop ***/

	for (int i = 0; i < this->Model.GetNt(); i++)
	  {
	    this->Model.Forward();
	    // Goes back.
	    this->Model.StepBack();
	  }

	this->OutputSaver.Save(this->Model);
	date.AddSeconds(Delta_t);
      }
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_DRIVER_STATIONARYDRIVER_CXX
#endif
