// Copyright (C) 2005-2007 CEREA
//     Author(s): Edouard Debry
//
// CEREA (http://www.enpc.fr/cerea/) is a joint laboratory of ENPC
// (http://www.enpc.fr/) and EDF R&D (http://www.edf.fr/).
//
// This file is part of the air quality modeling system Polyphemus.
//
// Polyphemus is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// Polyphemus is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// For more information, please see the Polyphemus web site:
//     http://www.enpc.fr/cerea/polyphemus/


#ifndef POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITBACKUP_HXX


#include "BaseSaverUnit.cxx"


namespace Polyphemus
{


  //////////////
  // INCLUDES //
  //////////////


#include <vector>
#include <fstream>
  using namespace std;

#include "AtmoData.hxx"
  using namespace AtmoData;


  /////////////////////
  // SAVERUNITBACKUP //
  /////////////////////


  /*! \brief This class is used to perform a backup saving of concentrations 
    over the whole domain of the underlying model in order to restart it. */
  template<class T, class ClassModel>
  class SaverUnitBackup: public BaseSaverUnit<T, ClassModel>
  {

  protected:

    //! Type of saver.
    string type;

    //! List of output files.
    vector<string> output_file;

    //! Date file.
    string date_file;

  public:

    SaverUnitBackup();
    virtual ~SaverUnitBackup();
    
    virtual string GetType() const;
    virtual string GetGroup() const;

    virtual void Init(ConfigStream& config_stream, ClassModel& Model);
    virtual void InitStep(ClassModel& Model);
    virtual void Save(ClassModel& Model);

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITBACKUP_HXX
#endif
