// Copyright (C) 2005-2007 CEREA
//     Author(s): Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea/) is a joint laboratory of ENPC
// (http://www.enpc.fr/) and EDF R&D (http://www.edf.fr/).
//
// This file is part of the air quality modeling system Polyphemus.
//
// Polyphemus is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// Polyphemus is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// For more information, please see the Polyphemus web site:
//     http://www.enpc.fr/cerea/polyphemus/


#ifndef POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITDOMAIN_HXX


#include "BaseSaverUnit.cxx"


namespace Polyphemus
{


  //////////////
  // INCLUDES //
  //////////////


#include <vector>
#include <fstream>
  using namespace std;

#include "AtmoData.hxx"
  using namespace AtmoData;


  /////////////////////
  // SAVERUNITDOMAIN //
  /////////////////////


  /*! \brief This class is used to save concentrations over the whole domain
    of the underlying model. Chemical species and vertical levels may be
    selected. */
  template<class T, class ClassModel>
  class SaverUnitDomain: public BaseSaverUnit<T, ClassModel>
  {

  protected:

    //! Type of saver ("domain" or "domain_ensemble").
    string type;
    //! Ensemble mode?
    bool with_ensemble;

    //! List of vertical levels to be saved.
    vector<int> levels;
    //! Number of levels to be saved.
    int Nlevels;

    //! List of output files.
    vector<string> output_file;

    //! Date at which the previous call to method 'Save' has been performed.
    Date previous_date;
    
    //! Model number (for ensembles).
    int Nmodel;
    //! Total number of initial calls.
    int Ncall;

    //! Buffer used to compute averaged concentrations.
    Data<T, 4> Concentration_;

  public:

    SaverUnitDomain();
    virtual ~SaverUnitDomain();
    
    virtual string GetType() const;
    virtual string GetGroup() const;

    virtual void Init(ConfigStream& config_stream, ClassModel& Model);
    virtual void InitStep(ClassModel& Model);
    virtual void Save(ClassModel& Model);

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITDOMAIN_HXX
#endif
